@@ -20,6 +20,7 @@ SEXP r_compute_max_ig(
         SEXP Rin_range,
         SEXP Rin_pseudocount,
         SEXP Rin_interesting_vars,
+        SEXP Rin_require_all_vars,
         SEXP Rin_return_tuples,
         SEXP Rin_use_cuda)
 {
@@ -80,9 +81,9 @@ SEXP r_compute_max_ig(
     dataset.loadData(
         &rawdata,
         DiscretizationInfo(
-            asInteger(Rin_seed), 
-            discretizations, 
-            divisions, 
+            asInteger(Rin_seed),
+            discretizations,
+            divisions,
             asReal(Rin_range)
         )
     );
@@ -95,27 +96,30 @@ SEXP r_compute_max_ig(
         0.0f,
         INTEGER(Rin_interesting_vars),
         length(Rin_interesting_vars),
-        true // true in max IGs, false in all matching tuple
+        asLogical(Rin_require_all_vars)
     );
 
     SEXP Rout_max_igs = PROTECT(allocVector(REALSXP, variable_count));
     SEXP Rout_tuples = nullptr;
+    SEXP Rout_dids = nullptr;
 
     const bool return_tuples = asLogical(Rin_return_tuples);
     MDFSOutput mdfs_output(MDFSOutputType::MaxIGs, variable_count);
     if (return_tuples) {
         Rout_tuples = PROTECT(allocMatrix(INTSXP, mdfs_info.dimensions, variable_count));
-        mdfs_output.setMaxIGsTuples(INTEGER(Rout_tuples)); // tuples are set row-first during computation, we transpose the result in R to speed up C code
+        Rout_dids = PROTECT(allocVector(INTSXP, variable_count));
+        mdfs_output.setMaxIGsTuples(INTEGER(Rout_tuples), INTEGER(Rout_dids)); // tuples are set row-first during computation, we transpose the result in R to speed up C code
     }
 
-    scalarMDFS(mdfs_info, &dataset, mdfs_output);
+    mdfs[asInteger(Rin_dimensions)-1](mdfs_info, &dataset, mdfs_output);
 
     mdfs_output.copyMaxIGsAsDouble(REAL(Rout_max_igs));
 
     int result_members_count = 1;
 
     if (return_tuples) {
-        result_members_count += 1;
+        result_members_count += 1; // for tuples
+        result_members_count += 1; // for disc nr
     }
 
     SEXP Rout_result = PROTECT(allocVector(VECSXP, result_members_count));
@@ -123,6 +127,7 @@ SEXP r_compute_max_ig(
 
     if (return_tuples) {
         SET_VECTOR_ELT(Rout_result, 1, Rout_tuples);
+        SET_VECTOR_ELT(Rout_result, 2, Rout_dids);
     }
 
     UNPROTECT(1 + result_members_count);
@@ -141,6 +146,7 @@ SEXP r_compute_all_matching_tuples(
         SEXP Rin_range,
         SEXP Rin_pseudocount,
         SEXP Rin_interesting_vars,
+        SEXP Rin_require_all_vars,
         SEXP Rin_ig_thr)
 {
     const int* dataDims = INTEGER(getAttrib(Rin_data, R_DimSymbol));
@@ -158,9 +164,9 @@ SEXP r_compute_all_matching_tuples(
     dataset.loadData(
         &rawdata,
         DiscretizationInfo(
-            asInteger(Rin_seed), 
-            discretizations, 
-            divisions, 
+            asInteger(Rin_seed),
+            discretizations,
+            divisions,
             asReal(Rin_range)
         )
     );
@@ -173,12 +179,12 @@ SEXP r_compute_all_matching_tuples(
         asReal(Rin_ig_thr),
         INTEGER(Rin_interesting_vars),
         length(Rin_interesting_vars),
-        false // true in max IGs, false in all matching tuple
+        asLogical(Rin_require_all_vars)
     );
 
     MDFSOutput mdfs_output(MDFSOutputType::MatchingTuples, variable_count);
 
-    scalarMDFS(mdfs_info, &dataset, mdfs_output);
+    mdfs[asInteger(Rin_dimensions)-1](mdfs_info, &dataset, mdfs_output);
 
     const int result_members_count = 3;
     const int tuples_count = mdfs_output.getMatchingTuplesCount();
@@ -198,3 +204,37 @@ SEXP r_compute_all_matching_tuples(
 
     return Rout_result;
 }
+
+#include "cpu/discretize.h"
+#include <vector>
+#include <algorithm>
+
+extern "C"
+SEXP r_discretize(
+        SEXP Rin_variable,
+        SEXP Rin_variable_idx,
+        SEXP Rin_divisions,
+        SEXP Rin_discretization_nr,
+        SEXP Rin_seed,
+        SEXP Rin_range)
+{
+    const R_len_t obj_count = length(Rin_variable);
+    const int discretization_nr = asInteger(Rin_discretization_nr);
+    const int variable_idx = asInteger(Rin_variable_idx);
+    const int divisions = asInteger(Rin_divisions);
+    const int seed = asInteger(Rin_seed);
+    double range = asReal(Rin_range);
+    double* variable = REAL(Rin_variable);
+
+    std::vector<double> sorted_variable(variable, variable + obj_count);
+    std::sort(sorted_variable.begin(), sorted_variable.end());
+
+    uint8_t* discretized_variable = new uint8_t[obj_count];
+    discretize(seed, discretization_nr, variable_idx, divisions, obj_count, variable, sorted_variable, discretized_variable, range);
+    SEXP Rout_result = PROTECT(allocVector(INTSXP, obj_count));
+    std::copy(discretized_variable, discretized_variable + obj_count, INTEGER(Rout_result));
+    delete[] discretized_variable;
+    UNPROTECT(1);
+
+    return Rout_result;
+}
