@@ -11,6 +11,7 @@ SEXP r_compute_max_ig(
 	SEXP Rin_range,
 	SEXP Rin_pseudocount,
 	SEXP Rin_interesting_vars,
+	SEXP Rin_require_all_vars,
 	SEXP Rin_return_tuples,
 	SEXP Rin_use_cuda
 );
@@ -26,5 +27,16 @@ extern "C"
 	SEXP Rin_range,
 	SEXP Rin_pseudocount,
 	SEXP Rin_interesting_vars,
+	SEXP Rin_require_all_vars,
 	SEXP Rin_ig_thr
 );
+
+extern "C"
+	SEXP r_discretize(
+	SEXP Rin_variable,
+	SEXP Rin_variable_nr,
+	SEXP Rin_divisions,
+	SEXP Rin_discretization_nr,
+	SEXP Rin_seed,
+	SEXP Rin_range
+);
